# encoding: utf-8

require "restclient_api_base/version"
require "json"
require "rest-client"
require "lumberjack"

# For other framework except Rails
begin
  require "active_support/concern"
rescue LoadError => e
  require "concern"
end

module RestclientApiBase
  extend ActiveSupport::Concern

  module ClassMethods

    # base_url: the api base url, like: http://api.example.com, default is ''
    # private_params: your api secret, like: { app_id: 'xxxx' }, default is {}

    attr_accessor :base_url, :private_params, :logger_name, :debug

    def get api_url, options = {}
      url    = base_url + api_url
      params = private_params.merge(options)

      puts "method: GET, url: #{url}, params: #{params}" if debug

      begin
        res = RestClient.get(url, params: params)
      rescue => e
        e.response
      end
    end

    def post api_url, query = {}, headers = {}
      url    = base_url + api_url
      params = private_params.merge(query)

      puts "method: POST, url: #{url}, params: #{params}" if debug

      begin
        res = RestClient.post(url, JSON.generate(params), content_type.merge(headers))
      rescue => e
        e.response
      end
    end

    def patch api_url, query = {}, headers = {}
      url    = base_url + api_url
      params = private_params.merge(query)

      puts "method: PATCH, url: #{url}, params: #{params}" if debug

      begin
        res = RestClient.patch(url, JSON.generate(params), content_type.merge(headers))
      rescue => e
        e.response
      end
    end

    def put api_url, query = {}, headers = {}
      url    = base_url + api_url
      params = private_params.merge(query)

      puts "method: PUT, url: #{url}, params: #{params}" if debug

      begin
        res = RestClient.put(url, JSON.generate(params), content_type.merge(headers))
      rescue => e
        e.response
      end
    end

    def delete api_url, options = {}
      url    = base_url + api_url
      params = private_params.merge(options)

      puts "method: DELETE, url: #{url}, params: #{params}" if debug

      begin
        res = RestClient.delete(url, params)
      rescue => e
        e.response
      end
    end

    def base_url
      @base_url ||= ''
    end

    def content_type
      { content_type: :json, accept: :json }
    end

    def private_params
      @private_params ||= {}
    end

    def private_params= params
      raise 'the private params must be a hash' unless params.is_a? Hash
      @private_params = params
    end

    def logger_name
      @logger_name ||= self.to_s.downcase.gsub(/::/, '_')
    end

    def write_log *args
      logger  = Lumberjack::Logger.new("log/#{logger_name}.log")
      logger.info(args.join('-'))
    end

    alias :write_api_log :write_log
  end
end
